var gulp = require('gulp');
var gulpSequence = require('gulp-sequence');
var pump = require('pump');
var through = require('through');
var sloc = require('gulp-sloc');

var sass = require('gulp-sass');
var minifyCss = require('gulp-clean-css');

var htmlmin = require('gulp-htmlmin');
var templateCache = require('gulp-angular-templatecache');
var gettext = require('gulp-angular-gettext');

var concat = require('gulp-concat');
var uglify = require('gulp-uglify');
var ngAnnotate = require('gulp-ng-annotate');
var clean = require('gulp-clean');

var jshint = require('gulp-jshint');
var htmlhint = require('gulp-htmlhint');

var all = ['!**/node_modules/**/*.*', '!**/bower_components/**/*.*', '!**/.*/*.*', '!**/.*/**/*.*'];
var client = ['src/js/index.js', 'src/js/templates.js', 'src/js/translations.js', 'src/js/**/*.js', '!**/.svn/*.*', '!**/.svn/**/*.*'];

//main tasks
gulp.task('default', ['hint', 'build']);

gulp.task('hint', function (callback) {
  gulpSequence('jshint', 'htmlhint', 'sloc', callback);
});

gulp.task('build', function (callback) {
  gulpSequence(['cache', 'translations', 'sass'], ['js-concat', 'js-min', 'css-min'], callback);
});

//js tasks
gulp.task('cache', function (callback) {
  pump([
    gulp.src(all.concat('src/js/**/*.html')),
    htmlmin({ collapseWhitespace: true }),
    templateCache('templates.js', { module: 'zapi' }),
    gulp.dest('src/js')
  ], callback);
});

gulp.task('js-concat', function (callback) {
  pump([
    gulp.src(client),
    concat('alphascript-client.js'),
    ngAnnotate(),
    gulp.dest('dist')
  ], callback);
});
gulp.task('js-min', function (callback) {
  pump([
    gulp.src(client),
    concat('alphascript-client.min.js'),
    ngAnnotate(),
    uglify(),
    gulp.dest('dist')
  ], callback);
});

//css tasks
gulp.task('sass', function (callback) {
  pump([
    gulp.src('src/css/*.scss'),
    sass().on('error', sass.logError),
    gulp.dest('dist')
  ], callback);
});
gulp.task('css-min', function (callback) {
  pump([
    gulp.src('dist/alphascript-client.css'),
    concat('alphascript-client.min.css'),
    minifyCss(),
    gulp.dest('dist')
  ], callback);
});

//lang tasks
gulp.task('pot', ['pot:clean'], function (callback) {
  pump([
    gulp.src(['src/js/**/*.html', 'src/js/**/*.js']),
    gettext.extract('template.pot', {
      extensions: {
        html: 'html',
        js: 'html'
      }
    }),
    gulp.dest('lang')
  ], callback);
});

gulp.task('pot:clean', function (callback) {
  pump([
    gulp.src('src/js/translations.js', {read: false}),
    clean()
  ], callback);
});

gulp.task('translations', function (callback) {
  pump([
    gulp.src('lang/*.po'),
    gettext.compile({
      module: "zapi"
    }),
    concat('translations.js'),
    gulp.dest('src/js')
  ], callback);
});

//lint tasks
gulp.task('htmlhint', function (callback) {
  pump([
    gulp.src(all.concat(['src/js/*.html', 'src/js/**/*.html'])),
    htmlhint("assets/gulp/htmlhintrc"),
    htmlhint.reporter("htmlhint-stylish"),
    count('html files found')
  ], callback);
});

gulp.task('jshint', function (callback) {
  pump([
    gulp.src(all.concat(['*.js', '**/*.js', '!*.min.js', '!**/*.min.js'])),
    jshint("assets/gulp/jshintrc"),
    jshint.reporter('jshint-stylish'),
    count('js files found')
  ], callback);
});

gulp.task('sloc', function (callback) {
  pump([
    gulp.src(all.concat(['*.js', '**/*.js', '!*.min.js', '!**/*.min.js'])),
    sloc()
  ], callback);
});

function count(message) {
  var count = 0;

  function countFiles(file) {
    count++;
  }

  function endStream() {
    console.log(count + ' ' + message || 'files processed.');
    this.emit('end');
  }

  return through(countFiles, endStream);
}