app.directive('entityList', function () {
  return {
    restrict: 'E',
    scope: {
      entity: '@',
      items: '=?',
      lookup: '=?',
      noTable: '@'
    },
    templateUrl: 'directives/entity/entity.list.html',
    controller: function ($scope, $interpolate, $controller, $entity, $stateParams, $rootScope, $interval, zapi, blockUI, NgTableParams) {
      //validate
      $scope.entity = $scope.entity || $stateParams.entity;
      $scope.entityObj = zapi.entityMap[$scope.entity];
      if (!$scope.entityObj) return;
      if (!$scope.entityObj.list) return;

      //defaults
      var idField = $scope.entityObj.id || "_id";
      $scope.items = $scope.items || [];
      if (typeof $scope.lookup === 'undefined') $scope.lookup = {};
      $scope.noTable = $scope.noTable || false;
      if (!$scope.noTable) $scope.table = { params: null };
      $scope.contentUrl = 'entity/' + $scope.entity + '/' + $scope.entity + '.list.html';

      //main
      initialize(loadFinished);

      //private functions
      function initialize(callback) {
        blockUI.start($interpolate("{{'api.entity.getData' | translate}}")($scope));
        //get main entity
        $entity.get($scope.entity).then(function (response) {
          $scope.items = response.data;
          if (!$scope.noTable) $scope.table.params = new NgTableParams({}, { dataset: $scope.items });

          //get lookup entities
          var entities = ($scope.entityObj.lookup || []);
          $entity.lookupMany(entities, function (err, lookups) {
            blockUI.stop();
            callback();
            if (err) return swal("Não foi possível obter os dados", err, 'warning');

            if (typeof $scope.lookup === 'undefined') $scope.lookup = {};
            for (var key in lookups) $scope.lookup[key] = lookups[key];
          });
        }).catch(function (response) {
          console.error(response);
        });
      }

      function loadFinished() {
        var intervalId = $interval(function () {
          //get websocket channel
          var websocket;
          try {
            if (typeof $scope.entityObj.websocket !== 'function') websocket = $scope.entity;
            else websocket = $scope.entityObj.websocket($rootScope, $stateParams) + "." + $scope.entity;

            if (typeof websocket !== 'string') return;
          } catch (err) {
            return;
          }
          $interval.cancel(intervalId);

          //listen to websocket channel
          $scope.$on('socket:' + websocket + '.remove', onThisEntityRemove);
          $scope.$on('socket:' + websocket + '.list.new', onThisEntityNew);
          $scope.$on('socket:' + websocket + '.list.edit', onThisEntityEdit);

          //inject child controller
          try {
            $controller($scope.entity + 'Ctrl', { $scope: $scope });
          } catch (err) {
            console.log(err);
          }
        }, 100);
      }

      function onThisEntityNew(ev, data) {
        //todo
        //$scope.lookup[$scope.entity] = $lookup.set($scope.entity, data);
        //$scope.items = angular.copy($lookup.get($scope.entity));
        $scope.items.unshift(data);
        if ($scope.noTable) return;

        $scope.table.params.total($scope.items.length);
        $scope.table.params.reload();
        //$scope.table.params = new NgTableParams({}, { dataset: $scope.items });
      }

      function onThisEntityEdit(ev, data) {
        //todo
        //$scope.lookup[$scope.entity] = $lookup.set($scope.entity, data);
        //$scope.items = angular.copy($lookup.get($scope.entity));
        var index = $scope.items.findIndex(function (item) {
          return data[idField] == item[idField];
        });
        if (index === -1) return;
        $scope.items[index] = data;

        if ($scope.noTable) return;

        $scope.table.params.total($scope.items.length);
        $scope.table.params.reload();
        //$scope.table.params = new NgTableParams({}, { dataset: $scope.items });
      }

      function onThisEntityRemove(ev, id) {
        //todo
        //$scope.lookup[$scope.entity] = $lookup.unset($scope.entity, id);
        //$scope.items = angular.copy($lookup.get($scope.entity));

        var index = $scope.items.findIndex(function (item) {
          return item[idField] == id;
        });
        if (index !== -1) $scope.items.splice(index, 1);
        if ($scope.noTable) return;

        $scope.table.params.total($scope.items.length);
        $scope.table.params.reload();
        //$scope.table.params = new NgTableParams({}, { dataset: $scope.items });
      }
    }
  };
});