app.directive('predefinedList', function () {
  return {
    restrict: 'A',
    scope: {
      listname: '@?',
      db: '@?'
    },
    replace: false,
    templateUrl: 'directives/list/list.html',
    controller: function ($scope, $list, $page, NgTableParams, $stateParams, $httpParamSerializer, $timeout, $location, $interpolate, $modal, zapi) {
      var db = $scope.db || $stateParams.db || "common";

      $scope.lists = [];
      $scope.list = { name: "", parameters: [], values: {} };
      $scope.tableParams = null;
      $scope.items = [];
      $scope.isGroupHeaderRowVisible = true;

      $scope.get = function (list) {
        if (!list) return;

        $list.get(list, db, function (items, cols) {
          //set columns
          if (typeof cols !== 'undefined') list.cols = cols;
          list.cols = setColumns(list.cols);

          var initialParams = {};
          if (list.group) {
            if (list.group.multi === true) {
              var groupByCustom = function (item) {
                for (var i = 0; i < list.group.items.length; i++) {
                  list.group.formatGroup = list.group.formatGroup.replace('{' + i + '}', item[list.group.items[i]]);
                }
                return list.group.formatGroup;
              };
              groupByCustom.title = list.group.title;
              groupByCustom.sortDirection = list.group.sort;
              initialParams.group = groupByCustom;
              $scope.isGroupHeaderRowVisible = (list.group.showGroupHeader === true);
            } else { initialParams.group = list.group; }
          }

          var params = { dataset: items };
          if (typeof list.groupOptions !== 'undefined') params.groupOptions = list.groupOptions;

          $scope.items = items;
          $scope.tableParams = new NgTableParams(initialParams, params);
        });
      };

      $scope.reset = function (list) {
        if (!list) return;

        $scope.items = [];
        $scope.tableParams = null;
        var currentLocation = $location.url().replace($scope.listname, "");
        if (!currentLocation.endsWith('/')) currentLocation += '/';
        $location.url(currentLocation + list.name);
      };

      $scope.setParameters = function (list) {
        if (!list) return;

        $modal.open('predefinedListParameters', 'md', list, function (list) {
          $scope.list = list;
          $scope.get(list);
        }, function (cancelReason) {
          //dismissed
        });
      };

      $scope.applySearch = function (search) {
        if (!search) return;

        var criteria = angular.copy(search.text);
        if (search.inverted) criteria = "!" + criteria;
        $scope.tableParams.filter({ $: criteria });
      };

      $scope.getExcelUrl = function (list) {
        if (!list) return;

        return '/api/list/values/' + db + '/' + list.name + '?xls=1&' + $httpParamSerializer({ query: list.values, dbs: $scope.selectedDbs });
      };

      $scope.getPdfUrl = function (list) {
        if (!list) return;

        return '/api/list/values/' + db + '/' + list.name + '?pdf=1&' + $httpParamSerializer({ query: list.values, dbs: $scope.selectedDbs });
      };

      $timeout(initialize);

      function initialize() {
        //selecionar automáticamente caso seja apenas 1 db
        $list.getMetadata(function (lists) {
          $scope.lists = lists;

          //selecionar automáticamente a lista no URL
          if (typeof $scope.listname === 'undefined') return;
          var list = lists.find(function (list) {
            return list.name === $scope.listname;
          });
          if (!list) return;

          //set columns
          list.cols = setColumns(list.cols);

          $scope.list = list;
          $page.setTitle(list.caption);
          if (list.parameters.length === 0) $scope.get(list);
          else $scope.setParameters(list);
        });
      }

      function setColumns(cols) {
        return cols.map(function (col) {
          col.getValue = interpolatedValue;
          if (col.interpolateExpr) {
            col.interpolateExpr = $interpolate(col.interpolateExpr);
          } else {
            col.interpolateExpr = $interpolate("{{row." + col.field + "}}");
          }
          return col;
        });
      }

      function interpolatedValue($scope, row) {
        return this.interpolateExpr({ row: row });
      }
    }
  };
});