app.controller('recordNotificationCtrl', function ($scope, $uibModalInstance, $http, zapi, toastr, data) {
  var config = zapi.entityMap[data.entity] && zapi.entityMap[data.entity].notification;
  if (!config) return $uibModalInstance.dismiss();
  $scope.config = config;

  $scope.cancel = $uibModalInstance.dismiss;
  $scope.ok = $uibModalInstance.close;

  $scope.entity = data.entity;
  $scope.record = data.record;

  $scope.notification = {
    recordEvent: data.notifications.find(function (n) {
      return n.type === 'recordEvent' && n.entity === data.entity && n.docId === data.record[config.fields.docId];
    }) || {
        type: 'recordEvent',
        entity: data.entity,

        label: data.record[config.fields.label],
        docId: data.record[config.fields.docId],
      },

    recordDate: config.fields.dates.map(function (date) {
      var recordDate = data.notifications.find(function (n) {
        return n.type === 'recordDate' && n.entity === data.entity && n.docId === data.record[config.fields.docId] && n.field === date.field;
      });
      if (recordDate) recordDate.active = true;
      else recordDate = {
        type: 'recordDate',
        entity: data.entity,
        active: false,

        label: data.record[config.fields.label],
        docId: data.record[config.fields.docId],

        fieldLabel: date.label,
        field: date.field,
        daysNotice: -10,
      };
      return recordDate;
    }),
  };

  $scope.setNotification = function (notification) {
    var url = '/api/notification/set';
    if (notification.type === 'recordDate' && !notification.active) url = '/api/notification/unset';

    $http.post(url, notification).then(function (response) {
      var method;
      var message;
      switch (notification.type) {
        case "recordEvent":
          if (!notification.eventAdd && !notification.eventEdit && !notification.eventRemove) {
            method = 'error';
            message = "Subscrição anulada";
          }
          else {
            method = 'success';
            message = 'Subscrição por evento registada';
          }
          break;

        case "recordDate":
          if (notification.active) {
            method = 'success';
            message = 'Subscrição por data registada';
          }
          else {
            method = 'error';
            message = 'Subscrição anulada';
          }
          break;

        default:
          console.error('unknown notification type', notification.type);
      }

      toastr[method]('Notificação', message);
    }).catch(function (response) {
      swal("Não foi possível guardar a subscrição", response.data, "warning");
    });
  };
});