app.factory('$auth', function ($http, $crypto, $location, anonymousPages, $uibModalStack, customPages, routeDepth, $rootScope, Idle, zapi, $license, blockUI, apiError) {
  var separator = '#/';

  function getUrlPaths(url) {
    var start = url.indexOf(separator) + separator.length;
    var end = getUrlEnd(url, start);
    var shortURL = start >= separator.length ? (end >= 0 ? url.substring(start, end) : url.substring(start)) : "";
    var urlPaths = shortURL.split('/');
    if (!(urlPaths instanceof Array)) urlPaths = [];
    return urlPaths;
  }

  function getUrlEnd(url, start) {
    var querystring = url.indexOf('?');
    if (querystring >= 0) {
      //using querystring
      return querystring;
    }
    else {
      //not using querystring
      var anchor = url.lastIndexOf('#');
      if (anchor === start - separator.length) {
        //not using anchor
        return -1;
      }
      else {
        //using anchor
        return anchor;
      }
    }
  }

  var service = {
    currentUser: null,

    login: function (username, password) {
      return $http.post(zapi.serverUrl + '/api/session/login/', {
        username: username,
        password: $crypto.isMd5(password) ? password : $crypto.md5(password)
      }).then(function (response) {
        if (zapi.idle) Idle.watch();
        service.currentUser = response.data.user;
      });
    },

    logout: function (callback) {
      return $http.get(zapi.serverUrl + '/api/session/logout').then(function (response) {
        //logout user
        service.currentUser = null;
        $rootScope.login = {};
        //unwatch idle
        if (zapi.idle) Idle.unwatch();
        //close all swal
        swal.close();
        //close all modals
        $uibModalStack.dismissAll();
        //close all blockUI
        blockUI.reset();
        //redirect to root path
        $location.path('/');

        if (typeof callback === 'function') callback();
      });
    },

    requestCurrentUser: function (callback) {
      if (zapi.idle) Idle.watch();
      if (service.isAuthenticated()) return callback(null, service.currentUser);

      $http.get(zapi.serverUrl + '/api/session/currentuser').then(function (response) {
        service.currentUser = response.data.user;
        callback(null, service.currentUser);
      }).catch(function (response) {
        service.currentUser = null;
        $rootScope.login = {};
        callback(response.data || apiError.timeout);
      });
    },

    isAuthenticated: function () {
      return !!service.currentUser;
    },

    activateAccount: function (codes) {
      return $http.post(zapi.serverUrl + '/api/user/activate/', { email: codes[0], code: codes[1] });
    },

    resetPassword: function (email, username) {
      return $http.post(zapi.serverUrl + '/api/user/resetpassword/', { email: email, username: username });
    },

    changePassword: function (user, newPassword1, newPassword2) {
      user.newPassword = $crypto.md5(newPassword1);
      return $http.post(zapi.serverUrl + '/api/user/changepassword/', user);
    },

    isPageAnonymous: function (urlPaths) {
      if (!urlPaths) urlPaths = getUrlPaths($location.path());

      return anonymousPages.some(function (anonymousPage) {
        return anonymousPage === urlPaths[0];
      });
    },

    isPageCustom: function (urlPaths) {
      if (!urlPaths) urlPaths = getUrlPaths($location.path());

      return customPages.find(function (customPage) {
        if (customPage.urlParts.length < urlPaths.length) return false;
        return !customPage.urlParts.some(function (part, i) {
          //ignore url wildcards by configuring the custom page with %
          if (part === '%') return false;
          return part !== urlPaths[i];
        });
      });
    },

    getNextRoute: function (callback) {
      return function (event, next, current) {
        var urlPaths = getUrlPaths(next);
        if (urlPaths.length === 0) return callback(undefined, undefined, event, next, current);

        if (service.isPageAnonymous(urlPaths)) return callback(undefined, undefined, event, next, current);

        //service.currentUser = null;
        service.requestCurrentUser(function (err, user) {
          $rootScope.login = (err ? {} : user) || {};

          if (!$rootScope.login._id) return callback("Sem sessão iniciada. Isto pode ocorrer por motivos de inactividade, aceder à mesma conta noutro computador ou actualização recente ao servidor.", user, event, next, current);

          if (urlPaths[0] === 'profile') return callback(undefined, user, event, next, current);

          //authenticate and authorize custom pages
          var customPage = service.isPageCustom(urlPaths);
          if (customPage) {
            //check license
            if (typeof customPage.license !== 'undefined') {
              if (!$license.isLicensed(customPage.license, service.currentUser)) $license.notify(customPage.license);
            }

            return callback(undefined, user, event, next, current);
          }

          //authenticate and authorize entity pages
          var entity = zapi.entityMap[urlPaths[routeDepth]];
          if (!entity) return callback("A página que está a tentar consultar não existe", user, event, next, current);
          if (urlPaths.length <= routeDepth + 1) return callback("A página que está a tentar consultar não existe", user, event, next, current);
          var action = entity[urlPaths[routeDepth + 1]];
          if (!action) return callback("A página que está a tentar consultar não existe", user, event, next, current);
          if (user.role.admin) return callback(undefined, user, event, next, current);
          if (action.admin) return callback("A página que está a tentar consultar não existe", user, event, next, current);

          //check license
          if (typeof entity.license !== 'undefined') {
            if (!$license.isLicensed(entity.license, service.currentUser)) $license.notify(entity.license);
          }
          return callback(undefined, user, event, next, current);
        });
      };
    },

    isAnonymous: function () {
      return !service.isLoggedIn();
    },

    isLoggedIn: function () {
      if (!service.currentUser) return false;
      return !!service.currentUser._id;
    },

    isAdmin: function () {
      if (service.isLoggedIn()) return service.currentUser.role.admin;
      return false;
    },

    hasClearance: function (level) {
      level = level || 0;
      if (!service.currentUser) return false;
      if (!service.currentUser.role) return false;
      return service.currentUser.role.approvalLevel >= level;
    }
  };

  return service;
});