app.service('$entity', function ($http, $q, mySocket, $rootScope, $stateParams, $timeout, $util, $interpolate, $location, $httpParamSerializer, blockUI, zapi, apiError) {
  var listeners = {};
  var self = this;

  this.get = function (entity, id) {
    var entityObj = zapi.entityMap[entity];

    var db = typeof entityObj.db === "function" ? entityObj.db($stateParams) : (entity.db || "common");
    var url = zapi.serverUrl + '/api/' + entity + '/list/' + db + (id ? '/' + id : '');

    if (!listeners[entity]) {
      listeners[entity] = true;
      $timeout(function () {
        var websocket;
        if (typeof entityObj.websocket !== 'function') websocket = entity;
        else websocket = entityObj.websocket($rootScope, $stateParams) + "." + entity;

        mySocket.forward(websocket + '.remove');
        mySocket.forward(websocket + '.new');
        mySocket.forward(websocket + '.edit');
        mySocket.forward(websocket + '.list.new');
        mySocket.forward(websocket + '.list.edit');
      });
    }

    return $http.get(url + '?' + $httpParamSerializer($location.search()));
  };

  this.getMany = function (entityKeys, callback) {
    //validate
    if (typeof entityKeys === 'string') entityKeys = [entityKeys];
    if (!(entityKeys instanceof Array)) return callback(undefined, {});
    if (entityKeys.length === 0) return callback(undefined, {});
    try {
      entityKeys.some(function (entityKey) {
        if (typeof entityKey !== 'string') throw "Não é possível obter dados de " + entityKey;
      });
    } catch (err) {
      return callback(err, {});
    }

    //assemble promises
    var promises = entityKeys.map(function (entityKey) {
      if (!listeners[entityKey]) {
        var entityObj = zapi.entityMap[entityKey];
        listeners[entityKey] = true;
        $timeout(function () {
          var websocket;
          if (typeof entityObj.websocket !== 'function') websocket = entityKey;
          else websocket = entityObj.websocket($rootScope, $stateParams) + "." + entityKey;

          mySocket.forward(websocket + '.remove');
          mySocket.forward(websocket + '.new');
          mySocket.forward(websocket + '.edit');
          mySocket.forward(websocket + '.list.new');
          mySocket.forward(websocket + '.list.edit');
        });
      }

      return self.get(entityKey);
    });

    //resolve promises
    $q.all(promises).then(function (results) {
      //todo return from cache
      var lookups = {};
      results.forEach(function (result, i) {
        lookups[entityKeys[i]] = result.data;
      });

      callback(null, lookups);
    }).catch(function (response) {
      callback(response.data || apiError.timeout);
    });
  };

  this.add = function (entity, item, scope) {
    return $q(function (resolve, reject) {
      var entityObj = zapi.entityMap[entity];

      var beforeSave = typeof entityObj.beforeSave === 'function' ? entityObj.beforeSave : $util.skip;
      beforeSave(item, function (err) {
        if (err) {
          swal("Antes de gravar...", err, "info");
          return reject(err);
        }

        var db = typeof entityObj.db === "function" ? entityObj.db($stateParams) : (entity.db || "common");
        var id = entityObj.id || "_id";
        var url = zapi.serverUrl + '/api/' + entity + '/add/' + db + (item[id] ? '/' + item[id] : '');

        blockUI.start($interpolate("{{'api.entity.sendData' | translate}}")(scope));
        var fd = new FormData();
        $util.setFormFiles(fd, item);
        fd.append('item', angular.toJson(item));

        $http.post(url, fd, { transformRequest: angular.identity, headers: { 'Content-Type': undefined } }).then(function (response) {
          if (item[id]) swal("Alterado", $interpolate("{{'api.entity.newRecordSuccess'|translate}}")(scope), "success");
          else swal("Criado", $interpolate("{{'api.entity.editRecordSuccess' | translate}}")(scope), "success");

          var afterSave = typeof entityObj.afterSave === 'function' ? entityObj.afterSave : $util.skip;
          afterSave($location, $rootScope);

          resolve(response);
        }).catch(function (response) {
          swal("Erro ao Guardar", response.data || apiError.timeout, "error");

          reject(response);
        }).finally(function () {
          blockUI.stop();
        });
      });
    });
  };

  this.update = function (entity, id, fieldsToSet) {
    var entityObj = zapi.entityMap[entity];

    var db = typeof entityObj.db === "function" ? entityObj.db($stateParams) : (entity.db || "common");
    var url = zapi.serverUrl + "/api/" + entity + "/update/" + db + '/' + id;

    return $http.post(url, fieldsToSet);
  };

  this.remove = function (entity, item, scope) {
    var deferred = $q.defer();
    var entityObj = zapi.entityMap[entity];

    blockUI.start($interpolate("{{'api.entity.checkRemove' | translate}}")(scope));

    var db = typeof entityObj.db === "function" ? entityObj.db($stateParams) : (entity.db || "common");
    var id = entityObj.id || "_id";
    var url = zapi.serverUrl + '/api/' + entity + '/remove/' + db + '/' + item[id];

    $http.get(url).then(function (response) {
      swal("Removido", $interpolate("{{'" + entityObj.title + "'|translate}}")(scope) + " - " + $interpolate("{{'api.entity.deleteRecordSuccess' | translate}}")(scope), "success");

      deferred.resolve(response);
    }).catch(function (response) {
      swal("Erro ao Remover", response.data || apiError.timeout, "warning");

      deferred.reject(response);
    }).finally(function () {
      blockUI.stop();
    });

    return deferred.promise;
  };

  this.lookup = function (entity) {
    var entityObj = zapi.entityMap[entity];

    var db = typeof entityObj.db === "function" ? entityObj.db($stateParams) : (entity.db || "common");
    var url = zapi.serverUrl + '/api/' + entity + '/lookup/' + db;

    return $http.get(url + '?' + $httpParamSerializer($location.search()));
  };

  this.lookupMany = function (entityKeys, callback) {
    //validate
    if (typeof entityKeys === 'string') entityKeys = [entityKeys];
    if (!(entityKeys instanceof Array)) return callback(undefined, {});
    if (entityKeys.length === 0) return callback(undefined, {});
    try {
      entityKeys.some(function (entityKey) {
        if (typeof entityKey !== 'string') throw "Não é possível obter dados de " + entityKey;
      });
    } catch (err) {
      return callback(err, {});
    }

    //assemble promises
    var promises = entityKeys.map(function (entityKey) {
      return self.lookup(entityKey);
    });

    //resolve promises
    $q.all(promises).then(function (results) {
      //todo return from cache
      var lookups = {};
      results.forEach(function (result, i) {
        lookups[entityKeys[i]] = result.data;
      });

      callback(null, lookups);
    }).catch(function (response) {
      callback(response.data || apiError.timeout);
    });
  };
});